# Import
from apm import *
	
# Select server
server = 'http://byu.apmonitor.com'

# Clear previous application
apm(server,'clear all')

# Load model file
apm_load(server,'cstr.apm')

# Load time points for future predictions
csv_load(server,'cstr.csv')

# Load replay replay data for local use
data = csv.reader(open('control.csv', 'rb'))
#data = csv.reader(open('replay1.csv', 'rb'))
#data = csv.reader(open('replay2.csv', 'rb'))
#data = csv.reader(open('replay3.csv', 'rb'))
replay = []
for row in data:
	replay.append(row)
len_replay = len(replay)

# APM Variable Classification
# class = FV, MV, SV, CV
#   F or FV = Fixed value - parameter may change to a new value every cycle
#   M or MV = Manipulated variable - independent variable over time horizon
#   S or SV = State variable - model variable for viewing
#   C or CV = Controlled variable - model variable for control

#Parameters
FVs = 'v','rho','cp','mdelh','eoverr','k0','ua'
MVs = 'tc','q','caf','tf'

#Variables
SVs = 'ca','---'
CVs = 't','---'

# Set up variable classifications for data flow
for x in FVs: apm_info(server,'FV',x)
for x in MVs: apm_info(server,'MV',x)
for x in SVs: apm_info(server,'SV',x)
for x in CVs: apm_info(server,'CV',x)

# Options

# time units (1=sec,2=min,3=hrs,etc)
apm_option(server,'nlc.ctrl_units',2)
apm_option(server,'nlc.hist_units',2)

# set controlled variable error model type
apm_option(server,'nlc.cv_type',2)
apm_option(server,'nlc.ev_type',2)
apm_option(server,'nlc.reqctrlmode',3)

# read discretization from CSV file
apm_option(server,'nlc.csv_read',1)

# turn on historization to see past results
apm_option(server,'nlc.hist_hor',100)

# set web plot update frequency
apm_option(server,'nlc.web_plot_freq',1)

# set number of nodes
apm_option(server,'nlc.nodes',3)

# Objective for Nonlinear Control

# Controlled variable (c)
apm_option(server,'t.sp',335)
apm_option(server,'t.sphi',345)
apm_option(server,'t.splo',335)
apm_option(server,'t.tau',1.0)
apm_option(server,'t.status',1)
apm_option(server,'t.fstatus',0)

# Manipulated variables (u)
apm_option(server,'tc.upper',400)
apm_option(server,'tc.dmax',10)
apm_option(server,'tc.lower',0)
apm_option(server,'tc.status',1)
apm_option(server,'tc.fstatus',0)

# imode (1=ss, 2=mpu, 3=rto, 4=sim, 5=mhe, 6=nlc)
apm_option(server,'nlc.imode',1)
solver_output = apm(server,'solve')
apm_option(server,'nlc.imode',6)

for isim in range(1, len_replay-1):
	print ('')
	print ('--- Cycle %i of %i ---' %(isim,len_replay))

	# allow server to process other requests
	time.sleep(3.0)

	for x in FVs:
		value = csv_element(x,isim,replay)
		if (not math.isnan(value)):
			response = apm_meas(server,x,value)
			print response
	for x in MVs:
		value = csv_element(x,isim,replay)
		if (not math.isnan(value)):
			response = apm_meas(server,x,value)
			print response
	for x in CVs:
		value = csv_element(x,isim,replay)
		if (not math.isnan(value)):
			response = apm_meas(server,x,value)
			print response

	# schedule a set point change at cycle 40
	#if (isim==4): apm_option(server,'volume.sp',50)

	# Run NLC on APM server
	solver_output = apm(server,'solve')
	print(solver_output)

	if (isim==1):
		# Open Web Viewer and Display Link
		print "Opening web viewer"
		url = apm_web(server)

	# Retrieve results (MEAS,MODEL,NEWVAL)
	# MEAS = FV, MV,or CV measured values
	# MODEL = SV & CV predicted values
	# NEWVAL = FV & MV optimized values
